#include <stdlib.h>
#include <stdio.h>

#include <fftw3-mpi.h>
#include <omp.h>

int threads_ok;

int main(int argc, char **argv)
{
  ptrdiff_t N0, N1;
  fftw_plan plan;
  fftw_complex *data;
  ptrdiff_t alloc_local, local_n0, local_0_start, i, j;
  int provided;
  int rank, nranks;
  double t0, t1;
  
  if(argc < 10) {
    fprintf(stderr, "USAGE: %s <N>\n", argv[0]);
    exit(-1);
  }
  N0 = atoi(argv[1]);
  N1 = N0;

  MPI_Init_thread(&argc, &argv, MPI_THREAD_FUNNELED, &provided);
  threads_ok = provided >= MPI_THREAD_FUNNELED;
  if (threads_ok) threads_ok = fftw_init_threads();
  fftw_mpi_init();
  int nthreads = omp_get_max_threads();
  if (threads_ok) fftw_plan_with_nthreads(nthreads);

  MPI_Comm comm = MPI_COMM_WORLD;
  MPI_Comm_rank(comm, &rank);
  MPI_Comm_size(comm, &nranks);

  if(rank==0) {
    printf("\nomp_max_threads = %d\n",nthreads);
    printf("fftw_planner_nthreads = %d\n\n",fftw_planner_nthreads());
  }

  t0 = MPI_Wtime();
  /* get local data size and allocate */
  alloc_local = fftw_mpi_local_size_2d(N0, N1, comm,
				       &local_n0, &local_0_start);
  data = fftw_alloc_complex(alloc_local);
  t1 = MPI_Wtime();
  if(rank==0)printf("t_%s = %f seconds\n","alloc",t1-t0);
  
  t0 = MPI_Wtime();
  /* create plan for in-place forward DFT */
  plan = fftw_mpi_plan_dft_2d(N0, N1, data, data, comm,
			      FFTW_FORWARD, FFTW_ESTIMATE);
  t1 = MPI_Wtime();
  if(rank==0)printf("t_%s = %f seconds\n","plan",t1-t0);

  t0 = MPI_Wtime();
  /* initialize data to some function my_function(x,y) */
#pragma omp parallel for
  for (i = 0; i < local_n0; ++i)
    for (j = 0; j < N1; ++j) {
      data[i*N1 + j][0] = data[i*N1 + j][1] = 0.0;
      if( (local_0_start + i) == 0 && j == 0)
	data[i*N1 + j][0] = 1.0;
    }
  t1 = MPI_Wtime();
  if(rank==0)printf("t_%s = %f seconds\n","init",t1-t0);

  t0 = MPI_Wtime();
  /* compute transforms, in-place, as many times as desired */
  fftw_execute(plan);
  t1 = MPI_Wtime();
  if(rank==0)printf("t_%s = %f seconds\n","fftf",t1-t0);

  fftw_destroy_plan(plan);

  MPI_Finalize();
}
